@@ -55,6 +55,7 @@ static khash_t (si32) * ht_seqs        = NULL;
 static khash_t (ss32) * ht_hostnames   = NULL;
 static khash_t (si32) * ht_cnt_overall = NULL;
 static khash_t (iglp) * ht_last_parse  = NULL;
+static khash_t (ss32) * ht_json_logfmt = NULL;
 
 static GKHashModule *cache_storage = NULL;
 
@@ -782,7 +783,8 @@ ins_is32 (khash_t (is32) * hash, uint32_t key, char *value) {
 /* Insert a string key and the corresponding string value.
  * Note: If the key exists, the value is not replaced.
  *
- * On error, or if key exists, -1 is returned.
+ * On error, -1 is returned.
+ * If key exists, 1 is returned.
  * On success 0 is returned */
 static int
 ins_ss32 (khash_t (ss32) * hash, const char *key, const char *value) {
@@ -795,11 +797,16 @@ ins_ss32 (khash_t (ss32) * hash, const char *key, const char *value) {
 
   dupkey = xstrdup (key);
   k = kh_put (ss32, hash, dupkey, &ret);
-  /* operation failed, or key exists */
-  if (ret == -1 || ret == 0) {
+  /* operation failed */
+  if (ret == -1) {
     free (dupkey);
     return -1;
   }
+  /* key exists */
+  if (ret == 0) {
+    free (dupkey);
+    return 1;
+  }
 
   kh_val (hash, k) = xstrdup (value);
 
@@ -2660,6 +2667,38 @@ ht_insert_hostname (const char *ip, const char *host) {
   return ins_ss32 (hash, ip, host);
 }
 
+/* Insert a JSON log format specification such as request.method => %m.
+ *
+ * On error -1 is returned.
+ * On success or if key exists, 0 is returned */
+int
+ht_insert_json_logfmt (GO_UNUSED void *userdata, char *key, char *spec) {
+  int ret;
+  char *dupkey = NULL;
+  khash_t (ss32) * hash = ht_json_logfmt;
+  khint_t k;
+
+  if (!hash)
+    return -1;
+
+  k = kh_get (ss32, hash, key);
+  /* key found, free it then to insert */
+  if (k != kh_end (hash))
+    free (kh_val (hash, k));
+  else {
+    dupkey = xstrdup (key);
+    k = kh_put (ss32, hash, dupkey, &ret);
+    /* operation failed */
+    if (ret == -1) {
+      free (dupkey);
+      return -1;
+    }
+  }
+  kh_val (hash, k) = xstrdup (spec);
+
+  return 0;
+}
+
 GLastParse
 ht_get_last_parse (uint32_t key) {
   khash_t (iglp) * hash = ht_last_parse;
@@ -2851,6 +2890,20 @@ ht_get_hostname (const char *host) {
   return get_ss32 (hash, host);
 }
 
+/* Get the string value from ht_json_logfmt given a JSON specifier key.
+ *
+ * On error, NULL is returned.
+ * On success the string value for the given key is returned */
+char *
+ht_get_json_logfmt (const char *key) {
+  khash_t (ss32) * hash = ht_json_logfmt;
+
+  if (!hash)
+    return NULL;
+
+  return get_ss32 (hash, key);
+}
+
 /* Get the string value from ht_agent_vals (user agent) given an uint32_t key.
  *
  * On error, NULL is returned.
@@ -3307,6 +3360,11 @@ parse_raw_data (GModule module) {
   return raw_data;
 }
 
+void
+init_pre_storage (void) {
+  ht_json_logfmt = (khash_t (ss32) *) new_ss32_ht ();
+}
+
 /* Initialize hash tables */
 void
 init_storage (void) {
@@ -3353,6 +3411,7 @@ free_storage (void) {
   des_igkh (ht_dates);
   des_si32_free (ht_seqs, 1);
   des_ss32_free (ht_hostnames, 1);
+  des_ss32_free (ht_json_logfmt, 1);
 
   FOREACH_MODULE (idx, module_list) {
     module = module_list[idx];
