@@ -120,7 +120,9 @@ static const char *os[][2] = {
  * returned. */
 static char *
 get_real_android (const char *droid) {
-  if (strstr (droid, "10"))
+  if (strstr (droid, "11"))
+    return alloc_string ("Android 11");
+  else if (strstr (droid, "10"))
     return alloc_string ("Android 10");
   else if (strstr (droid, "9"))
     return alloc_string ("Pie 9");
@@ -202,7 +204,9 @@ get_real_win (const char *win) {
  * returned. */
 static char *
 get_real_mac_osx (const char *osx) {
-  if (strstr (osx, "10.15"))
+  if (strstr (osx, "11.0"))
+    return alloc_string ("macOS 11.0 Big Sur");
+  else if (strstr (osx, "10.15"))
     return alloc_string ("macOS 10.15 Catalina");
   else if (strstr (osx, "10.14"))
     return alloc_string ("macOS 10.14 Mojave");
@@ -268,7 +272,6 @@ parse_ios (char *agent, int tlen) {
   char *p = NULL, *q = NULL;
   ptrdiff_t offset;
 
-  p = agent;
   if ((p = strstr (agent, " OS ")) == NULL)
     goto out;
 
