@@ -37,6 +37,7 @@
 #define _FILE_OFFSET_BITS 64
 
 #define _XOPEN_SOURCE 700
+#define _DEFAULT_SOURCE
 
 #include <ctype.h>
 #include <errno.h>
@@ -66,10 +67,12 @@
 #include "parser.h"
 
 #include "browsers.h"
-#include "goaccess.h"
 #include "error.h"
+#include "goaccess.h"
 #include "opesys.h"
+#include "pdjson.h"
 #include "util.h"
+#include "websocket.h"
 #include "xmalloc.h"
 
 /* private prototypes */
@@ -925,28 +928,77 @@ parse_req (char *line, char **method, char **protocol) {
   return dreq;
 }
 
-/* Extract the next delimiter given a log format and copy the
- * delimiter(s) to the destination buffer.
- * Note that it's possible to store up to two delimiters.
- *
- * On error, the function returns.
- * On success, the delimiter(s) are stored in the dest buffer and the
- * number of extra delimiters is returned. */
+#if defined(HAVE_LIBSSL) && defined(HAVE_CIPHER_STD_NAME)
 static int
+extract_tls_version_cipher (char *tkn, char **cipher, char **tls_version) {
+  SSL_CTX *ctx = NULL;
+  SSL *ssl = NULL;
+  int code = 0;
+  unsigned short code_be;
+  unsigned char cipherid[3];
+  const SSL_CIPHER *c = NULL;
+  char *bEnd;
+  const char *sn = NULL;
+
+  code = strtoull (tkn, &bEnd, 10);
+  if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE) {
+    LOG_DEBUG (("unable to convert cipher code to a valid decimal."));
+    free (tkn);
+    return 1;
+  }
+
+  /* ssl context */
+  if (!(ctx = SSL_CTX_new (SSLv23_server_method ()))) {
+    LOG_DEBUG (("Unable to create a new SSL_CTX_new to extact TLS."));
+    free (tkn);
+    return 1;
+  }
+  if (!(ssl = SSL_new (ctx))) {
+    LOG_DEBUG (("Unable to create a new instace of SSL_new to extact TLS."));
+    free (tkn);
+    return 1;
+  }
+
+  code_be = htobe16 (code);
+  memcpy (cipherid, &code_be, 2);
+  cipherid[2] = 0;
+
+  if (!(c = SSL_CIPHER_find (ssl, cipherid))) {
+    LOG_DEBUG (("Unable to find cipher to extact TLS."));
+    free (tkn);
+    return 1;
+  }
+
+  if (!(sn = SSL_CIPHER_standard_name (c))) {
+    LOG_DEBUG (("Unable to get cipher standard name to extact TLS."));
+    free (tkn);
+    return 1;
+  }
+  *cipher = xstrdup (sn);
+  *tls_version = xstrdup (SSL_CIPHER_get_version (c));
+
+  free (tkn);
+  SSL_free (ssl);
+  SSL_CTX_free (ctx);
+
+  return 0;
+}
+#endif
+
+/* Extract the next delimiter given a log format and copy the delimiter to the
+ * destination buffer.
+ *
+ * On error, the dest buffer will be empty.
+ * On success, the delimiter(s) are stored in the dest buffer. */
+static void
 get_delim (char *dest, const char *p) {
   /* done, nothing to do */
   if (p[0] == '\0' || p[1] == '\0') {
     dest[0] = '\0';
-    return 0;
+    return;
   }
   /* add the first delim */
   dest[0] = *(p + 1);
-  /* check if there's another possible delim */
-  if (p[2] == '|' && p[3] != '%' && p[3] != '\0') {
-    dest[1] = *(p + 3);
-    return 1;
-  }
-  return 0;
 }
 
 /* Extract and malloc a token given the parsed rule.
@@ -1009,6 +1061,21 @@ find_alpha (char **str) {
   *str += s - *str;
 }
 
+/* Move forward through the log string until a non-space (!isspace)
+ * char is found and returns the count. */
+static int
+find_alpha_count (char *str) {
+  int cnt = 0;
+  char *s = str;
+  while (*s) {
+    if (isspace (*s))
+      s++, cnt++;
+    else
+      break;
+  }
+  return cnt;
+}
+
 /* Format the broken-down time tm to a numeric date format.
  *
  * On error, or unable to format the given tm, 1 is returned.
@@ -1053,21 +1120,16 @@ spec_err (GLogItem * logitem, int code, const char spec, const char *tkn) {
   const char *fmt = NULL;
 
   switch (code) {
-  case SPEC_TOKN_INV:
-    fmt = "Token '%s' doesn't match specifier '%%%c'";
-    err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-"), spec) + 1);
-    sprintf (err, fmt, (tkn ? tkn : "-"), spec);
-    break;
-  case SPEC_TOKN_SET:
-    fmt = "Token already set for '%%%c' specifier.";
-    err = xmalloc (snprintf (NULL, 0, fmt, spec) + 1);
-    sprintf (err, fmt, spec);
-    break;
   case SPEC_TOKN_NUL:
     fmt = "Token for '%%%c' specifier is NULL.";
     err = xmalloc (snprintf (NULL, 0, fmt, spec) + 1);
     sprintf (err, fmt, spec);
     break;
+  case SPEC_TOKN_INV:
+    fmt = "Token '%s' doesn't match specifier '%%%c'";
+    err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-"), spec) + 1);
+    sprintf (err, fmt, (tkn ? tkn : "-"), spec);
+    break;
   case SPEC_SFMT_MIS:
     fmt = "Missing braces '%s' and ignore chars for specifier '%%%c'";
     err = xmalloc (snprintf (NULL, 0, fmt, (tkn ? tkn : "-"), spec) + 1);
@@ -1076,7 +1138,7 @@ spec_err (GLogItem * logitem, int code, const char spec, const char *tkn) {
   }
   logitem->errstr = err;
 
-  return 1;
+  return code;
 }
 
 static void
@@ -1110,6 +1172,7 @@ set_numeric_date (uint32_t * numdate, const char *date) {
 static int
 parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end) {
   struct tm tm;
+  time_t now = time (0);
   const char *dfmt = conf.date_format;
   const char *tfmt = conf.time_format;
 
@@ -1117,18 +1180,27 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
   double serve_secs = 0.0;
   uint64_t bandw = 0, serve_time = 0;
   long status = 0L;
+  int dspc = 0, fmtspcs = 0;
 
   errno = 0;
   memset (&tm, 0, sizeof (tm));
+  localtime_r (&now, &tm);
 
   switch (*p) {
     /* date */
   case 'd':
     if (logitem->date)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
-    /* parse date format including dates containing spaces,
-     * i.e., syslog date format (Jul 15 20:10:56) */
-    if (!(tkn = parse_string (&(*str), end, count_matches (dfmt, ' ') + 1)))
+      return 0;
+
+    /* Attempt to parse date format containing spaces,
+     * i.e., syslog date format (Jul\s15, Nov\s\s2).
+     * Note that it's possible a date could contain some padding, e.g.,
+     * Dec\s\s2 vs Nov\s22, so we attempt to take that into consideration by looking
+     * ahead the log string and counting the # of spaces until we find an alphanum char. */
+    if ((fmtspcs = count_matches (dfmt, ' ')) && (pch = strchr (*str, ' ')))
+      dspc = find_alpha_count (pch);
+
+    if (!(tkn = parse_string (&(*str), end, MAX (dspc, fmtspcs) + 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
     if (str_to_time (tkn, dfmt, &tm) != 0 || set_date (&logitem->date, tm) != 0) {
@@ -1136,6 +1208,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       free (tkn);
       return 1;
     }
+
     set_numeric_date (&logitem->numdate, logitem->date);
     set_tm_dt_logitem (logitem, tm);
     free (tkn);
@@ -1143,7 +1216,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* time */
   case 't':
     if (logitem->time)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1152,13 +1225,14 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
       free (tkn);
       return 1;
     }
+
     set_tm_tm_logitem (logitem, tm);
     free (tkn);
     break;
     /* date/time as decimal, i.e., timestamps, ms/us  */
   case 'x':
     if (logitem->time && logitem->date)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1171,11 +1245,12 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     set_numeric_date (&logitem->numdate, logitem->date);
     set_tm_dt_logitem (logitem, tm);
     set_tm_tm_logitem (logitem, tm);
+    free (tkn);
     break;
     /* Virtual Host */
   case 'v':
     if (logitem->vhost)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
@@ -1184,7 +1259,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* remote user */
   case 'e':
     if (logitem->userid)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
@@ -1193,7 +1268,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* cache status */
   case 'C':
     if (logitem->cache_status)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL)
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
@@ -1205,7 +1280,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* remote hostname (IP only) */
   case 'h':
     if (logitem->host)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1219,7 +1294,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* request method */
   case 'm':
     if (logitem->method)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1233,7 +1308,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* request not including method or protocol */
   case 'U':
     if (logitem->req)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL || *tkn == '\0') {
       free (tkn);
@@ -1250,7 +1325,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* query string alone, e.g., ?param=goaccess&tbm=shop */
   case 'q':
     if (logitem->qstr)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     tkn = parse_string (&(*str), end, 1);
     if (tkn == NULL || *tkn == '\0') {
       free (tkn);
@@ -1267,7 +1342,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* request protocol */
   case 'H':
     if (logitem->protocol)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1281,7 +1356,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* request, including method + protocol */
   case 'r':
     if (logitem->req)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1291,10 +1366,16 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* Status Code */
   case 's':
     if (logitem->status)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
+    /* do not validate HTTP status code */
+    if (conf.no_strict_status) {
+      logitem->status = tkn;
+      break;
+    }
+
     status = strtol (tkn, &sEnd, 10);
     if (tkn == sEnd || *sEnd != '\0' || errno == ERANGE || status < 100 || status > 599) {
       spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
@@ -1306,7 +1387,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* size of response in bytes - excluding HTTP headers */
   case 'b':
     if (logitem->resp_size)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1320,7 +1401,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* referrer */
   case 'R':
     if (logitem->ref)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
 
     if (!(tkn = parse_string (&(*str), end, 1)))
       tkn = alloc_string ("-");
@@ -1346,7 +1427,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     /* user agent */
   case 'u':
     if (logitem->agent)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
+      return 0;
 
     tkn = parse_string (&(*str), end, 1);
     if (tkn != NULL && *tkn != '\0') {
@@ -1425,20 +1506,30 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
   case 'k':
     /* error to set this twice */
     if (logitem->tls_cypher)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
-
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
+#if defined(HAVE_LIBSSL) && defined(HAVE_CIPHER_STD_NAME)
+    {
+      char *tmp = NULL;
+      for (tmp = tkn; isdigit (*tmp); tmp++);
+      if (!strlen (tmp))
+        extract_tls_version_cipher (tkn, &logitem->tls_cypher, &logitem->tls_type);
+      else
+        logitem->tls_cypher = tkn;
+    }
+#else
     logitem->tls_cypher = tkn;
+#endif
+
     break;
 
     /* UMS: Krypto (TLS) parameters like "TLSv1.2" */
   case 'K':
     /* error to set this twice */
     if (logitem->tls_type)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
-
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1449,8 +1540,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
   case 'M':
     /* error to set this twice */
     if (logitem->mime_type)
-      return spec_err (logitem, SPEC_TOKN_SET, *p, NULL);
-
+      return 0;
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
@@ -1518,16 +1608,22 @@ static int
 find_xff_host (GLogItem * logitem, char **str, char **p) {
   char *ptr = NULL, *tkn = NULL, *skips = NULL;
   int invalid_ip = 1, len = 0, type_ip = TYPE_IPINV;
+  int idx = 0, skips_len = 0;
 
   if (!(skips = extract_braces (p)))
     return spec_err (logitem, SPEC_SFMT_MIS, **p, "{}");
 
+  skips_len = strlen (skips);
   ptr = *str;
   while (*ptr != '\0') {
     if ((len = strcspn (ptr, skips)) == 0) {
-      len++, ptr++;
+      len++, ptr++, idx++;
       goto move;
     }
+    /* If our index does not match the number of delimiters and we have already a
+     * valid client IP, then we assume we have reached the length of the XFF */
+    if (idx < skips_len && logitem->host)
+      break;
 
     ptr += len;
     /* extract possible IP */
@@ -1545,6 +1641,7 @@ find_xff_host (GLogItem * logitem, char **str, char **p) {
       logitem->type_ip = type_ip;
     }
     free (tkn);
+    idx = 0;
 
   move:
     *str += len;
@@ -1566,7 +1663,7 @@ special_specifier (GLogItem * logitem, char **str, char **p) {
     /* XFF remote hostname (IP only) */
   case 'h':
     if (logitem->host)
-      return spec_err (logitem, SPEC_TOKN_SET, **p, NULL);
+      return 0;
     if (find_xff_host (logitem, str, p))
       return spec_err (logitem, SPEC_TOKN_NUL, 'h', NULL);
     break;
@@ -1581,10 +1678,10 @@ special_specifier (GLogItem * logitem, char **str, char **p) {
  * On success, the malloc'd token is assigned to a GLogItem member and
  * 0 is returned. */
 static int
-parse_format (GLogItem * logitem, char *str) {
+parse_format (GLogItem * logitem, char *str, char *lfmt) {
   char end[2 + 1] = { 0 };
-  char *lfmt = conf.log_format, *p = NULL;
-  int perc = 0, tilde = 0, optdelim = 0;
+  char *p = NULL;
+  int perc = 0, tilde = 0, ret = 0;
 
   if (str == NULL || *str == '\0')
     return 1;
@@ -1616,13 +1713,10 @@ parse_format (GLogItem * logitem, char *str) {
         return 0;
 
       memset (end, 0, sizeof end);
-      optdelim = get_delim (end, p);
+      get_delim (end, p);
       /* attempt to parse format specifiers */
-      if (parse_specifier (logitem, &str, p, end) == 1)
-        return 1;
-      /* account for the extra delimiter */
-      if (optdelim)
-        p++;
+      if ((ret = parse_specifier (logitem, &str, p, end)))
+        return ret;
       perc = 0;
     } else if (perc && isspace (p[0])) {
       return 1;
@@ -2907,6 +3001,26 @@ process_invalid (GLog * glog, GLogItem * logitem, const char *line) {
     count_process_and_invalid (glog, line);
 }
 
+static int
+parse_json_specifier (void *ptr_data, char *key, char *str) {
+  GLogItem *logitem = (GLogItem *) ptr_data;
+  char *spec = NULL;
+  int ret = 0;
+
+  if (!(spec = ht_get_json_logfmt (key)) || 0 == strlen (str))
+    return 0;
+
+  ret = parse_format (logitem, str, spec);
+  free (spec);
+
+  return ret;
+}
+
+static int
+parse_json_format (GLogItem * logitem, char *str) {
+  return parse_json_string (logitem, str, parse_json_specifier);
+}
+
 /* Process a line from the log and store it accordingly taking into
  * account multiple parsing options prior to setting data into the
  * corresponding data structure.
@@ -2916,14 +3030,21 @@ int
 pre_process_log (GLog * glog, char *line, int dry_run) {
   GLogItem *logitem;
   int ret = 0;
+  char *fmt = conf.log_format;
 
   /* soft ignore these lines */
   if (valid_line (line))
     return -1;
 
   logitem = init_log_item (glog);
+
   /* Parse a line of log, and fill structure with appropriate values */
-  if ((ret = parse_format (logitem, line)) || (ret = verify_missing_fields (logitem))) {
+  if (conf.is_json_log_format)
+    ret = parse_json_format (logitem, line);
+  else
+    ret = parse_format (logitem, line, fmt);
+
+  if (ret || (ret = verify_missing_fields (logitem))) {
     process_invalid (glog, logitem, line);
     goto cleanup;
   }
