@@ -41,6 +41,7 @@
 #define MAX_IGNORE_STATUS      64
 #define MAX_OUTFORMATS          3
 #define MAX_FILENAMES         512
+#define MIN_DATENUM_FMT_LEN     7
 #define NO_CONFIG_FILE "No config file used"
 
 typedef enum LOGTYPE {
@@ -54,6 +55,7 @@ typedef enum LOGTYPE {
   CLOUDSTORAGE,
   AWSELB,
   AWSS3,
+  CADDY,
 } GLogType;
 
 /* predefined log times */
@@ -83,6 +85,7 @@ typedef struct GPreConfLog_ {
   const char *awselb;
   const char *squid;
   const char *awss3;
+  const char *caddy;
 } GPreConfLog;
 
 /* *INDENT-OFF* */
@@ -158,6 +161,7 @@ typedef struct GConf_
   int max_items;                    /* max number of items to output */
   int mouse_support;                /* add curses mouse support */
   int no_color;                     /* no terminal colors */
+  int no_strict_status;             /* don't enforce 100-599 status codes */
   int no_column_names;              /* don't show col names on termnal */
   int no_csv_summary;               /* don't show overall metrics */
   int no_html_last_updated;         /* don't show HTML last updated field */
@@ -172,6 +176,7 @@ typedef struct GConf_
   int real_time_html;               /* enable real-time HTML output */
   int restore;                      /* reload data from db-path */
   int skip_term_resolver;           /* no terminal resolver */
+  int is_json_log_format;           /* is a json log format */
   uint32_t keep_last;               /* number of days to keep in storage */
   uint32_t num_tests;               /* number of lines to test */
   uint64_t html_refresh;            /* refresh html report every X of seconds */
@@ -211,6 +216,8 @@ char *get_selected_date_str (size_t idx);
 char *get_selected_format_str (size_t idx);
 char *get_selected_time_str (size_t idx);
 const char *verify_formats (void);
+int is_json_log_format (const char *fmt);
+int parse_json_string (void *userdata, const char *str, int (*cb) (void *, char *, char *));
 size_t get_selected_format_idx (void);
 void set_date_format_str (const char *optarg);
 void set_log_format_str (const char *optarg);
