@@ -60,7 +60,7 @@
 #define SPIN_FMTM "[%s %s] {%'"PRIu64"} @ {%'lld/s}"
 #define SPIN_LBL 256    /* max length of the progress spinner */
 
-#define SPIN_UPDATE_INTERVAL 100000     // in microseconds
+#define SPIN_UPDATE_INTERVAL 100000000  /* in nanoseconds */
 
 /* Module JSON keys */
 #define VISITORS_ID        "visitors"
@@ -104,11 +104,11 @@
 
 /* CONFIG DIALOG */
 #define CONF_MENU_H       6
-#define CONF_MENU_W       57
+#define CONF_MENU_W       67
 #define CONF_MENU_X       2
 #define CONF_MENU_Y       4
 #define CONF_WIN_H        20
-#define CONF_WIN_W        61
+#define CONF_WIN_W        71
 
 /* FIND DIALOG */
 #define FIND_DLG_HEIGHT   8
@@ -145,11 +145,11 @@
 
 /* CONF ERROR DIALOG */
 #define ERR_MENU_HEIGHT   10
-#define ERR_MENU_WIDTH    57
+#define ERR_MENU_WIDTH    67
 #define ERR_MENU_X        2
 #define ERR_MENU_Y        4
 #define ERR_WIN_HEIGHT    15
-#define ERR_WIN_WIDTH     61
+#define ERR_WIN_WIDTH     71
 
 #include "color.h"
 #include "commons.h"
