@@ -52,6 +52,8 @@
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
 
+#define MAX(a,b) (((a)>(b))?(a):(b))
+
 /* *INDENT-OFF* */
 #include <stdint.h>
 #include <sys/types.h>
@@ -64,12 +66,10 @@ char *deblank (char *str);
 char *escape_str (const char *src);
 char *filesize_str (unsigned long long log_size);
 char *float2str (float d, int width);
-int str2int (const char *date);
 char *get_global_config (void);
 char *get_user_config (void);
 char *get_visitors_date (const char *odate, const char *from, const char *to);
 char *int2str (int d, int width);
-char *u322str (uint32_t d, int width);
 char *left_pad_str (const char *s, int indent);
 char *ltrim (char *s);
 char *replace_str (const char *str, const char *old, const char *new);
@@ -78,6 +78,8 @@ char *secs_to_str (int secs);
 char *strtoupper(char *str);
 char *substring (const char *str, int begin, int len);
 char *trim_str (char *str);
+char *u322str (uint32_t d, int width);
+char *u642str (uint64_t d, int width);
 char *unescape_str (const char *src);
 char *usecs_to_str (unsigned long long usec);
 const char *verify_status_code (char *str);
@@ -87,16 +89,17 @@ int count_matches (const char *s1, char c);
 int find_output_type (char **filename, const char *ext, int alloc);
 int hide_referer (const char *ref);
 int ignore_referer (const char *ref);
-int intlen (int num);
+int intlen (uint64_t num);
 int invalid_ipaddr (char *str, int *ipvx);
 int ip_in_range (const char *ip);
+int ptr2int(char *ptr);
+int str2int (const char *date);
 int str_inarray (const char *s, const char *arr[], int size);
 int str_to_time (const char *str, const char *fmt, struct tm *tm);
 int valid_output_type (const char *filename);
-int ptr2int(char *ptr);
 off_t file_size (const char *filename);
-uint32_t ip_to_binary (const char *ip);
 size_t append_str (char **dest, const char *src);
+uint32_t ip_to_binary (const char *ip);
 void genstr(char *dest, size_t len);
 void strip_newlines (char *str);
 void xstrncpy (char *dest, const char *source, const size_t dest_size);
